function FirstScanProjection(~, ~, dataset, mainAxes, mainWindow, perspective)
%FIRSTSCANPROJECTION Visualizes the projection of the first scan obtained 
%by the robot during one exploration

    [analyzedScans, datasetName] = processData(dataset,1);
    
    if perspective == 1
        plotOriginalScan(analyzedScans, mainAxes, mainWindow, datasetName);
    else
        plotRotatedScan(analyzedScans, mainAxes, mainWindow, datasetName);
    end
end

function [analyzedScans, datasetName] = processData(dataset, scanIndex)
    datasetName = dataset.Name;
    scan = dataset.data{scanIndex}.scan;
    LiDAR_Angle = dataset.LiDAR_Angle_Deg;
    LiDAR_Height = dataset.LiDAR_Height_mm;
    ceiling_Height = dataset.Ceiling_Height_mm;

    analyzedScans.scans = processScan(scan, LiDAR_Angle,...
        LiDAR_Height, 100, ceiling_Height);

    xGround = (-LiDAR_Height/sind(LiDAR_Angle));
    xGround = ones(1,2)*xGround;
    zGround = [-LiDAR_Height,-LiDAR_Height];
    yGround = [min(analyzedScans.scans.rotated.all.y), max(analyzedScans.scans.rotated.all.y)];

    analyzedScans.groundLine.original.x =  xGround;
    analyzedScans.groundLine.original.y =  yGround;
    analyzedScans.groundLine.original.z =  zGround;

    xGround = (-LiDAR_Height/sind(LiDAR_Angle))*sind(180+LiDAR_Angle-90);
    xGround = ones(1,2)*xGround;
    analyzedScans.groundLine.rotated.x =  xGround;
    analyzedScans.groundLine.rotated.y =  yGround;
    analyzedScans.groundLine.rotated.z =  zGround;
    analyzedScans.LiDAR_Angle_Deg = LiDAR_Angle;
end

function scans = processScan(scan, lidarAngle, lidarHeight, threshold, ceilHeight)
    [ang,dis] = cart2pol(scan.x,scan.y);
    
    removeIndexs = find(dis < 50);
    dis(removeIndexs) = [];
    ang(removeIndexs) = [];
    
    [X,Y] = pol2cart(ang,dis);
    ox = X;
    oy = Y;
    
    [rotX,rotY,rotZ] = rotatePoints(X, Y, zeros(1,length(X)),...
        [0, 0, lidarAngle]);
    
    [rWall,rGround,rCeiling] = removeGroundAndCeil(rotX, rotY, rotZ, ...
        ceilHeight-lidarHeight, threshold/2, -lidarHeight, threshold/2);
    
    
    % Rotated points
    scans.rotated.all.x = rotX;
    scans.rotated.all.y = rotY;
    scans.rotated.all.z = rotZ;
    
    scans.rotated.wall.x = rWall.x;
    scans.rotated.wall.y = rWall.y;
    scans.rotated.wall.z = rWall.z;
    
    scans.rotated.ground.x = rGround.x;
    scans.rotated.ground.y = rGround.y;
    scans.rotated.ground.z = rGround.z;
    
    scans.rotated.ceil.x = rCeiling.x;
    scans.rotated.ceil.y = rCeiling.y;
    scans.rotated.ceil.z = rCeiling.z;
    
    % Original Points
    scans.original.all.x = ox;
    scans.original.all.y = oy;
    scans.original.all.z = zeros(1,length(ox));
    
    % Original Points
    scans.original.all.x = ox;
    scans.original.all.y = oy;
    scans.original.all.z = zeros(1,length(ox));
    
    [rotX,rotY,rotZ] = rotatePoints(rWall.x, rWall.y, rWall.z,...
        [0, 0, -lidarAngle]);
    scans.original.wall.x = rotX;
    scans.original.wall.y = rotY;
    scans.original.wall.z = rotZ;
    
    [rotX,rotY,rotZ] = rotatePoints(rGround.x, rGround.y, rGround.z,...
        [0, 0, -lidarAngle]);
    scans.original.ground.x = rotX;
    scans.original.ground.y = rotY;
    scans.original.ground.z = rotZ;
    
    [rotX,rotY,rotZ] = rotatePoints(rCeiling.x, rCeiling.y, rCeiling.z,...
        [0, 0, -lidarAngle]);
    scans.original.ceil.x = rotX;
    scans.original.ceil.y = rotY;
    scans.original.ceil.z = rotZ;
end

function [wall, ground, ceiling] = removeGroundAndCeil(px, py, pz, ...
    ceilingHeight, ceilingTh, groundHeight, groundTh)

    groundIndx = find(pz < groundHeight+groundTh &...
        pz > groundHeight-groundTh);
    
    ceilingIndex = find(pz < ceilingHeight+ceilingTh & ...
        pz > ceilingHeight-ceilingTh);
    
    gx = px(groundIndx);
    gy = py(groundIndx);
    gz = pz(groundIndx);
    ground.x = gx; ground.y = gy; ground.z = gz;
    
    cx = px(ceilingIndex);
    cy = py(ceilingIndex);
    cz = pz(ceilingIndex);
    ceiling.x = cx; ceiling.y = cy; ceiling.z = cz;
    
    px([groundIndx,ceilingIndex]) = [];
    py([groundIndx,ceilingIndex]) = [];
    pz([groundIndx,ceilingIndex]) = [];
    wall.x = px; wall.y = py; wall.z = pz;
end



function plotOriginalScan(analyzedScans, plotAxes, figID, datasetName)
    figID.Name = [datasetName, ' (2D View)'];
    cla(plotAxes);
    
    hold(plotAxes,'on');
    wallPoints = analyzedScans.scans.original.wall;
    origWall = plot(wallPoints.x, wallPoints.y, '.','Color','b', ...
        'MarkerSize', 20, 'Parent',plotAxes);
    
    groundPoints = analyzedScans.scans.original.ground;
    origGround = plot(groundPoints.x, groundPoints.y, '.','Color',...
        [0,0.8,0], 'MarkerSize', 20, 'Parent', plotAxes);
    
    ceilingPoints = analyzedScans.scans.original.ceil;
    if ~isempty(ceilingPoints.x)
        origCeil = plot(ceilingPoints.x, ceilingPoints.y, '.', 'Color',...
            [0.8,0.8,0], 'MarkerSize', 20, 'Parent', plotAxes);
    else
        origCeil = [];
    end
    
    groundLine = analyzedScans.groundLine.original;
    origGL = plot(groundLine.x, groundLine.y, '--','Color','r',...
        'MarkerSize', 20,'LineWidth', 2, 'Parent',plotAxes);
    
    quiver(0,0,300,0,'Parent',plotAxes,'LineWidth',2,'Color','r',...
        'MaxHeadSize',1000);
    
    initPos = plot(0,0,'.','Color','r','MarkerSize',20,'Parent',plotAxes);
  
    hold(plotAxes,'off');
    axis(plotAxes,'equal');
    axis(plotAxes,'tight');
    grid(plotAxes,'on');
    
    plotAxes.XLabel.String = 'X (mm)';
    plotAxes.YLabel.String = 'Y (mm)';
    plotAxes.ZLabel.String = 'Z (mm)';
    
    if isempty(origCeil)
        legend(plotAxes,[initPos,origWall,origGround,origGL],{'Robot Position',...
            'Wall', 'Detected Ground',['Scan-Ground',char(10),'Intersection']});
    else
        legend(plotAxes,[initPos,origWall,origGround,origCeil,origGL],...
            {'Robot Position', 'Wall', 'Detected Ground','Ceiling Points',...
            ['Scan-Ground',char(10),'Intersection']});
    end
end

function plotRotatedScan(analyzedScans, plotAxes, figID, datasetName)
    figID.Name = [datasetName, ' (3D View)'];
    zLevel = analyzedScans.groundLine.rotated.z(1);
    cla(plotAxes);
    
    hold(plotAxes,'on');
    wallPoints = analyzedScans.scans.rotated.wall;
    origWall = plot3(wallPoints.x, wallPoints.y, wallPoints.z, '.', 'Color',...
        'b', 'MarkerSize', 20, 'Parent',plotAxes);
    
    plot3(wallPoints.x, wallPoints.y, ones(1,length(wallPoints.x))*zLevel, '.',...
        'Color', 'b', 'MarkerSize', 2, 'Parent',plotAxes);
    
    groundPoints = analyzedScans.scans.rotated.ground;
    origGround = plot3(groundPoints.x, groundPoints.y, groundPoints.z,...
        '.', 'Color', [0,0.8,0], 'MarkerSize', 20, 'Parent', plotAxes);
    
    ceilingPoints = analyzedScans.scans.original.ceil;
    if ~isempty(ceilingPoints.x)
        origCeil = plot3(ceilingPoints.x, ceilingPoints.y, ceilingPoints.z,...
            '.', 'Color', [0.8,0.8,0], 'MarkerSize', 20, 'Parent', plotAxes);
    else
        origCeil = [];
    end
    
    groundLine = analyzedScans.groundLine.rotated;
    origGL = plot3(groundLine.x, groundLine.y, groundLine.z, '--', 'Color',...
        'r', 'MarkerSize', 20, 'LineWidth', 2, 'Parent', plotAxes);
    
    xPoints = analyzedScans.scans.rotated.all.x;
    yPoints = analyzedScans.scans.rotated.all.y;
    xRange = [min(xPoints)-100,max(xPoints)+100,max(xPoints)+100,min(xPoints)-100];
    yRange = [max(yPoints)+100,max(yPoints)+100,min(yPoints)-100,min(yPoints)-100];
    zRange = ones(1,4)*analyzedScans.groundLine.rotated.z(1);
    
    plane = fill3(xRange,yRange,zRange,'c','Parent',plotAxes,'FaceAlpha',0.4,'EdgeColor','k');
    
    [qivx, quivy,quivz]  = rotatePoints(300,0,0,[0,0,analyzedScans.LiDAR_Angle_Deg]);
    quiver3(0,0,0,qivx,quivy,quivz,'Parent',plotAxes,'LineWidth',2,'Color',...
        'r','MaxHeadSize',1000);
    
    initPos = plot3(0,0,0,'.','Color','r','MarkerSize',20,'Parent',plotAxes);
    plot3(0,0,zLevel,'.','Color','r','MarkerSize',8,'Parent',plotAxes);
  
    quiver3(0,0,zLevel,300,0,0,'Parent',plotAxes,'LineWidth',1,'Color',...
        'r','MaxHeadSize',1000);
    
    if isempty(origCeil)
        legend(plotAxes,[initPos,origWall,origGround,origGL,plane],{'Robot Position',...
            'Wall', 'Detected Ground',['Scan-Ground',char(10),...
            'Intersection'],'Ground Plane'});
    else
        legend(plotAxes,[initPos,origWall,origGround,origCeil,origGL,plane],...
            {'Robot Position', 'Wall', 'Detected Ground', 'Ceiling Points',...
            ['Scan-Ground', char(10), 'Intersection'], 'Ground Plane'});
    end
    
    hold(plotAxes,'off');
    axis(plotAxes,'equal');
    axis(plotAxes,'tight');
    grid(plotAxes,'on');
    
    plotAxes.View = [33.2800   44.5600];
    plotAxes.XLabel.String = 'X (mm)';
    plotAxes.YLabel.String = 'Y (mm)';
    plotAxes.ZLabel.String = 'Z (mm)';
end

function [rotX,rotY,rotZ] = rotatePoints(px, py, pz, orientation)
    angleZ = orientation(1);
    angleX = orientation(2);
    angleY = orientation(3);

    Rx = [1,0,0;0,cosd(angleX),-sind(angleX);0,sind(angleX),cosd(angleX)];
    Ry = [cosd(angleY), 0,sind(angleY);0,1,0;-sind(angleY),0,cosd(angleY)];
    Rz = [cosd(angleZ),-sind(angleZ),0;sind(angleZ),cosd(angleZ),0;0,0,1];

    R = single(Rx*Ry*Rz);   

    rotX = zeros(1,length(px));
    rotY = zeros(1,length(py));
    rotZ = zeros(1,length(pz));
    
    for pointID = 1:length(rotX)
        rotatedPoint = [px(pointID),py(pointID),pz(pointID)]*R;
        rotX(pointID) = rotatedPoint(1);
        rotY(pointID) = rotatedPoint(2);
        rotZ(pointID) = rotatedPoint(3);
    end               
end