function FirstScanVisualizer(~, ~, dataset, axesID, mainWindow)
%FirstScanVisualizer Visualizes the first scan obtained by the robot during
%one exploration.
    [analyzedScans, datasetName] = processData(dataset, 1);
    
    plotScan(analyzedScans, axesID, mainWindow, datasetName);
end

function [analyzedScans, datasetName] = processData(dataset, scanIndex)
    datasetName = dataset.Name;

    scan = dataset.data{scanIndex}.scan;
    LiDAR_Angle = dataset.LiDAR_Angle_Deg;
    LiDAR_Height = dataset.LiDAR_Height_mm;
    ceiling_Height = dataset.Ceiling_Height_mm;

    analyzedScans = processScan(scan, LiDAR_Angle, LiDAR_Height, 100,...
        ceiling_Height);
end

function plotScan(analyzedScans, plotAxes, figID, datasetName)
    figID.Name = [datasetName, ' (First Scan)'];

    wallPoints = analyzedScans.original.wall;
    origWall = plot(wallPoints.x, wallPoints.y, '.', 'Color', 'b', ...
        'MarkerSize', 20, 'Parent', plotAxes);
    hold(plotAxes, 'on');
    groundPoints = analyzedScans.original.ground;
    origGround = plot(groundPoints.x, groundPoints.y, '.', 'Color',...
        [0, 0.8, 0], 'MarkerSize', 20, 'Parent', plotAxes);
    
    ceilingPoints = analyzedScans.original.ceil;
    if ~isempty(ceilingPoints.x)
        origCeil = plot(ceilingPoints.x, ceilingPoints.y, '.','Color',...
            [0.8, 0.8, 0], 'MarkerSize', 20, 'Parent', plotAxes);
    else
        origCeil = [];
    end
    
    quiver(0, 0, 1500, 0, 'Parent', plotAxes, 'LineWidth', 2, 'Color',...
        'r', 'MaxHeadSize', 10);
    initPos = plot(0, 0, '.', 'Color', 'r', 'MarkerSize', 20, 'Parent',...
        plotAxes);
  
    hold(plotAxes,'off');
    axis(plotAxes,'equal');
    axis(plotAxes,'tight');
    grid(plotAxes,'on');
    
    plotAxes.XLabel.String = 'X (mm)';
    plotAxes.YLabel.String = 'Y (mm)';
    plotAxes.ZLabel.String = 'Z (mm)';
    
    if isempty(origCeil)
        legend(plotAxes, [initPos, origWall, origGround], {'Robot Position',...
            'Wall Points', 'Ground Points'});
    else
        legend(plotAxes, [initPos, origWall, origGround, origCeil], ...
            {'Robot Position', 'Wall Points', 'Ground Points','Ceiling Points'});       
    end
end

function scans = processScan(scan, lidarAngle, lidarHeight, threshold, ceilHeight)
    [ang,dis] = cart2pol(scan.x,scan.y);
    
    removeIndexs = find(dis < 50);
    dis(removeIndexs) = [];
    ang(removeIndexs) = [];
    
    [X,Y] = pol2cart(ang,dis);
    ox = X;
    oy = Y;
    
    [rotX, rotY, rotZ] = rotatePoints(X, Y, zeros(1, length(X)),...
        [0, 0, lidarAngle]);
    
    [rWall,rGround,rCeiling] = removeGroundAndCeil(rotX, rotY, rotZ, ...
        ceilHeight-lidarHeight, threshold/2, -lidarHeight, threshold/2);
    
    % Rotated points
    scans.rotated.all.x = rotX;
    scans.rotated.all.y = rotY;
    scans.rotated.all.z = rotZ;
    
    scans.rotated.wall.x = rWall.x;
    scans.rotated.wall.y = rWall.y;
    scans.rotated.wall.z = rWall.z;
    
    scans.rotated.ground.x = rGround.x;
    scans.rotated.ground.y = rGround.y;
    scans.rotated.ground.z = rGround.z;
    
    scans.rotated.ceil.x = rCeiling.x;
    scans.rotated.ceil.y = rCeiling.y;
    scans.rotated.ceil.z = rCeiling.z;
    
    % Original Points
    scans.original.all.x = ox;
    scans.original.all.y = oy;
    scans.original.all.z = zeros(1,length(ox));
    
    [rotX,rotY,rotZ] = rotatePoints(rWall.x, rWall.y, rWall.z,...
        [0, 0, -lidarAngle]);
    scans.original.wall.x = rotX;
    scans.original.wall.y = rotY;
    scans.original.wall.z = rotZ;
    
    [rotX,rotY,rotZ] = rotatePoints(rGround.x, rGround.y, rGround.z,...
        [0, 0, -lidarAngle]);
    scans.original.ground.x = rotX;
    scans.original.ground.y = rotY;
    scans.original.ground.z = rotZ;
    
    [rotX,rotY,rotZ] = rotatePoints(rCeiling.x, rCeiling.y, rCeiling.z,...
        [0, 0, -lidarAngle]);
    scans.original.ceil.x = rotX;
    scans.original.ceil.y = rotY;
    scans.original.ceil.z = rotZ;
end

function [rotX,rotY,rotZ] = rotatePoints(px, py, pz, orientation)

    angleZ = orientation(1);
    angleX = orientation(2);
    angleY = orientation(3);

    Rx = [1,0,0;0,cosd(angleX),-sind(angleX);0,sind(angleX),cosd(angleX)];
    Ry = [cosd(angleY), 0,sind(angleY);0,1,0;-sind(angleY),0,cosd(angleY)];
    Rz = [cosd(angleZ),-sind(angleZ),0;sind(angleZ),cosd(angleZ),0;0,0,1];

    R = single(Rx*Ry*Rz);   

    rotX = zeros(1,length(px));
    rotY = zeros(1,length(py));
    rotZ = zeros(1,length(pz));
    
    for pointID = 1:length(rotX)
        rotatedPoint = [px(pointID),py(pointID),pz(pointID)]*R;
        rotX(pointID) = rotatedPoint(1);
        rotY(pointID) = rotatedPoint(2);
        rotZ(pointID) = rotatedPoint(3);
    end               
end

function [wall, ground, ceiling] = removeGroundAndCeil(px, py, pz, ...
    ceilingHeight, ceilingTh, groundHeight, groundTh)

    groundIndx = find(pz < groundHeight+groundTh & pz > groundHeight-groundTh);
    ceilingIndex = find(pz < ceilingHeight+ceilingTh & pz > ceilingHeight-ceilingTh);
    
    gx = px(groundIndx);
    gy = py(groundIndx);
    gz = pz(groundIndx);
    ground.x = gx; ground.y = gy; ground.z = gz;
    
    cx = px(ceilingIndex);
    cy = py(ceilingIndex);
    cz = pz(ceilingIndex);
    ceiling.x = cx; ceiling.y = cy; ceiling.z = cz;
    
    px([groundIndx,ceilingIndex]) = [];
    py([groundIndx,ceilingIndex]) = [];
    pz([groundIndx,ceilingIndex]) = [];
    wall.x = px; wall.y = py; wall.z = pz;
end