function LidarProjectionVisualizer(~, ~, dataset, mainAxes, mainWindow)
%FIRSTSCANPROJECTION Visualizes the projection of the first scan obtained 
%by the robot during one exploration
    [analyzedScans, datasetName] = processData(dataset, 1);
    
    plotRotatedScan(analyzedScans, datasetName, mainAxes, mainWindow);
end

function [analyzedScans, datasetName] = processData(dataset, scanIndex)

    datasetName = dataset.Name;

    scan = dataset.data{scanIndex}.scan;
    LiDAR_Angle = dataset.LiDAR_Angle_Deg;
    LiDAR_Height = dataset.LiDAR_Height_mm;
    ceiling_Height = dataset.Ceiling_Height_mm;
        
    analyzedScans.scans = processScan(scan, LiDAR_Angle,...
    LiDAR_Height, 100, ceiling_Height);

    xGround = (-LiDAR_Height/sind(LiDAR_Angle));
    xGround = ones(1,2)*xGround;
    zGround = [-LiDAR_Height,-LiDAR_Height];
    yGround = [min(analyzedScans.scans.rotated.all.y), ...
        max(analyzedScans.scans.rotated.all.y)];

    analyzedScans.groundLine.original.x =  xGround;
    analyzedScans.groundLine.original.y =  yGround;
    analyzedScans.groundLine.original.z =  zGround;

    xGround = (-LiDAR_Height/sind(LiDAR_Angle))*sind(180+LiDAR_Angle-90);
    xGround = ones(1,2)*xGround;
    analyzedScans.groundLine.rotated.x =  xGround;
    analyzedScans.groundLine.rotated.y =  yGround;
    analyzedScans.groundLine.rotated.z =  zGround;
    analyzedScans.LiDAR_Angle_Deg = LiDAR_Angle;
end

function scans = processScan(scan, lidarAngle, lidarHeight, threshold, ceilHeight)
    [ang,dis] = cart2pol(scan.x,scan.y);
    
    removeIndexs = find(dis < 50);
    dis(removeIndexs) = [];
    ang(removeIndexs) = [];
    
    [X,Y] = pol2cart(ang,dis);
    ox = X;
    oy = Y;
    
    [rotX,rotY,rotZ] = rotatePoints(X,Y,zeros(1,length(X)),[0,0,lidarAngle]);
    [rx,ry,rz,gx,gy,gz,cx,cy,cz] = removeGroundAndCeil(rotX,rotY,rotZ,...
        ceilHeight-lidarHeight, threshold/2,-lidarHeight,threshold/2);
    
    
    % Rotated points
    scans.rotated.all.x = rotX;
    scans.rotated.all.y = rotY;
    scans.rotated.all.z = rotZ;
    
    scans.rotated.wall.x = rx;
    scans.rotated.wall.y = ry;
    scans.rotated.wall.z = rz+lidarHeight;
    
    [ul,ol] = detectOversAndUnders(rx,ry,rz+lidarHeight);
    scans.rotated.wall.ul = ul;
    scans.rotated.wall.ol = ol;
    scans.rotated.wall.lidarHeight = lidarHeight;
    
    scans.rotated.ground.x = gx;
    scans.rotated.ground.y = gy;
    scans.rotated.ground.z = gz;
    
    scans.rotated.ceil.x = cx;
    scans.rotated.ceil.y = cy;
    scans.rotated.ceil.z = cz;
    
    % Original Points
    scans.original.all.x = ox;
    scans.original.all.y = oy;
    scans.original.all.z = zeros(1,length(ox));
    
    [rotX,rotY,rotZ] = rotatePoints(scans.rotated.wall.x,...
        scans.rotated.wall.y,scans.rotated.wall.z,[0,0,-lidarAngle]);
    
    scans.original.wall.x = rotX;
    scans.original.wall.y = rotY;
    scans.original.wall.z = rotZ;
    
    [rotX,rotY,rotZ] = rotatePoints(gx,gy,gz,[0,0,-lidarAngle]);
    scans.original.ground.x = rotX;
    scans.original.ground.y = rotY;
    scans.original.ground.z = rotZ;
    
    [rotX,rotY,rotZ] = rotatePoints(cx,cy,cz,[0,0,-lidarAngle]);
    scans.original.ceil.x = rotX;
    scans.original.ceil.y = rotY;
    scans.original.ceil.z = rotZ;
end

function [px,py,pz,gx,gy,gz,cx,cy,cz] = removeGroundAndCeil(px, py, pz,...
    ceilingHeight, ceilingTh, groundHeight, groundTh)

    removeIndx = find((pz < ceilingHeight+ceilingTh & pz > ceilingHeight-ceilingTh)...
        | (pz < groundHeight+groundTh & pz > groundHeight-groundTh) );
    
    groundIndx = find(pz < groundHeight+groundTh & pz > groundHeight-groundTh);
    ceilingIndex = find(pz < ceilingHeight+ceilingTh & pz > ceilingHeight-ceilingTh);
    
    gx = px(groundIndx);
    gy = py(groundIndx);
    gz = pz(groundIndx);
    
    cx = px(ceilingIndex);
    cy = py(ceilingIndex);
    cz = pz(ceilingIndex);
    
    px(removeIndx) = [];
    py(removeIndx) = [];
    pz(removeIndx) = [];
end

function plotRotatedScan(analyzedScans, datasetName, plotAxes, figID)
    figID.Name = [datasetName, ' (Projections)'];
    
    cla(plotAxes);
    hold(plotAxes,'on');
    wallPoints = analyzedScans.scans.rotated.wall;
    rotScan = plot3(wallPoints.x, -wallPoints.y, wallPoints.z, '.', 'Color',...
        'b', 'MarkerSize', 20, 'Parent',plotAxes);
    
    projWall = plot3(wallPoints.x, -wallPoints.y, zeros(1,length(wallPoints.x)), '.',...
        'Color', 'g', 'MarkerSize', 20, 'Parent',plotAxes);
    
    
    lh = analyzedScans.scans.rotated.wall.lidarHeight;
    
    
    fill3([wallPoints.x,0],-[wallPoints.y,0],[wallPoints.z,lh],'b',...
        'Parent',plotAxes,'FaceAlpha',0.3,'EdgeColor','b');
    
    fill3([wallPoints.x,0],-[wallPoints.y,0],zeros(1,length(wallPoints.x)+1),...
        'g','Parent',plotAxes,'FaceAlpha',0.3,'EdgeColor','g');
    
    [qivx, quivy,quivz]  = rotatePoints(300,0,0,[0,0,analyzedScans.LiDAR_Angle_Deg]);
    quiver3(0,0,lh,qivx,quivy,quivz,'Parent',plotAxes,'LineWidth',2,'Color'...
        ,'r','MaxHeadSize',1000);
    
    initPos = plot3(0,0,lh,'.','Color','r','MarkerSize',20,'Parent',plotAxes);
    plot3(0,0,0,'.','Color','r','MarkerSize',8,'Parent',plotAxes);
    
    xolp = analyzedScans.scans.rotated.wall.ol.x;
    yolp = analyzedScans.scans.rotated.wall.ol.y;
    zolp = analyzedScans.scans.rotated.wall.ol.z;
    overLines = plot3(xolp,-yolp,zolp,'c-','Parent', plotAxes);
    
    xulp = analyzedScans.scans.rotated.wall.ul.x;
    yulp = analyzedScans.scans.rotated.wall.ul.y;
    zulp = analyzedScans.scans.rotated.wall.ul.z;
    underLines = plot3(xulp,-yulp,zulp,'-','Color',[1,0,1],'Parent', plotAxes);
    
    quiver3(0,0,0,300,0,0,'Parent',plotAxes,'LineWidth',1,'Color','r',...
        'MaxHeadSize',1000);
    
    legend(plotAxes,[initPos,rotScan, projWall,overLines,underLines],...
        {'Robot Position', '3D Wall', '2D Projected Wall',...
        ['Projection Lines',char(10), '(Over Ground Level)'],...
        ['Projection Lines',char(10),'(Below Ground Level)']});

    hold(plotAxes,'off');
    axis(plotAxes,'equal');
    axis(plotAxes,'tight');
    grid(plotAxes,'on');
    box(plotAxes,'on');
    plotAxes.View = [33.2800   44.5600];
    plotAxes.XLabel.String = 'X (mm)';
    plotAxes.YLabel.String = 'Y (mm)';
    plotAxes.ZLabel.String = 'Z (mm)';
end

function [unders,overs] = detectOversAndUnders(nsX,nsY,nsZ)
    linealID = 1;

    unders.x = [];
    unders.y = [];
    unders.z = [];
    
    overs.x = [];
    overs.y = [];
    overs.z = [];
    
    for i = 1:3:length(nsX)*3
        
        if nsZ(linealID) < 0
            unders.x(end+1) = nsX(linealID);
            unders.x(end+1) = nsX(linealID);
            unders.x(end+1) = nan;

            unders.y(end+1) = nsY(linealID);
            unders.y(end+1) = nsY(linealID);
            unders.y(end+1) = nan;

            unders.z(end+1) = nsZ(linealID);
            unders.z(end+1) = 0;
            unders.z(end+1) = nan;
        else
            overs.x(end+1) = nsX(linealID);
            overs.x(end+1) = nsX(linealID);
            overs.x(end+1) = nan;

            overs.y(end+1) = nsY(linealID);
            overs.y(end+1) = nsY(linealID);
            overs.y(end+1) = nan;

            overs.z(end+1) = nsZ(linealID);
            overs.z(end+1) = 0;
            overs.z(end+1) = nan;            
        end
  
        linealID = linealID+1;
    end
end

function [rotX,rotY,rotZ] = rotatePoints(px, py, pz, orientation)
    angleZ = orientation(1);
    angleX = orientation(2);
    angleY = orientation(3);

    Rx = [1,0,0;0,cosd(angleX),-sind(angleX);0,sind(angleX),cosd(angleX)];
    Ry = [cosd(angleY), 0,sind(angleY);0,1,0;-sind(angleY),0,cosd(angleY)];
    Rz = [cosd(angleZ),-sind(angleZ),0;sind(angleZ),cosd(angleZ),0;0,0,1];

    R = single(Rx*Ry*Rz);   

    rotX = zeros(1,length(px));
    rotY = zeros(1,length(py));
    rotZ = zeros(1,length(pz));
    
    for pointID = 1:length(rotX)
        rotatedPoint = [px(pointID),py(pointID),pz(pointID)]*R;
        rotX(pointID) = rotatedPoint(1);
        rotY(pointID) = rotatedPoint(2);
        rotZ(pointID) = rotatedPoint(3);
    end               
end