function StaticObjectsVisualizer(~, ~, dataset, distanceID, mainAxes, mainWindow)
    %STATICOBJECTSVISUALIZER Creates a 3D representation of scanned objects
  
    [scans, objName, distance] = processData(dataset, distanceID);

    displayResult(scans, objName, distance, mainAxes, mainWindow);
end

function [scans, objName, distance] = processData(dataset, distanceID)
    anglesList = dataset.LiDAR_Angles_List_deg;
    numAngles = length(anglesList);
    objName = dataset.Name;
    distance = dataset.Distances_List_mm(distanceID);
    
    for angleID = 1:numAngles
        X = dataset.Distance{distanceID}.Angle{angleID}.Data.x;
        Y = dataset.Distance{distanceID}.Angle{angleID}.Data.y;
        Z = zeros(1,length(X));

        [rotX,rotY,rotZ] = rotatePoints(X,Y,Z,[0,0,anglesList(angleID)]);

        scans.Angle{angleID}.RotatedData.X = rotX;
        scans.Angle{angleID}.RotatedData.Y = rotY;
        scans.Angle{angleID}.RotatedData.Z = rotZ;
    end
end

function [rotX,rotY,rotZ] = rotatePoints(px, py, pz, orientation)
    angleZ = orientation(1);
    angleX = orientation(2);
    angleY = orientation(3);

    Rx = [1,0,0;0,cosd(angleX),-sind(angleX);0,sind(angleX),cosd(angleX)];
    Ry = [cosd(angleY), 0,sind(angleY);0,1,0;-sind(angleY),0,cosd(angleY)];
    Rz = [cosd(angleZ),-sind(angleZ),0;sind(angleZ),cosd(angleZ),0;0,0,1];

    R = single(Rx*Ry*Rz);   

    rotX = zeros(1,length(px));
    rotY = zeros(1,length(py));
    rotZ = zeros(1,length(pz));
    
    for pointID = 1:length(rotX)
        rotatedPoint = [px(pointID),py(pointID),pz(pointID)]*R;
        rotX(pointID) = rotatedPoint(1);
        rotY(pointID) = rotatedPoint(2);
        rotZ(pointID) = rotatedPoint(3);
    end   
end

function displayResult(scans, objName, distance, mainAxes, mainWindow)
    
    if distance == -1
        mainWindow.Name = [objName,' Not Present']; 
    else
        mainWindow.Name = [objName,' ',...
            num2str(distance),' mm'];        
    end
    
    colsStep = 1/length(scans.Angle);
    
    cla(mainAxes);
    hold(mainAxes,'on');
    
    for angleID = 1:length(scans.Angle)
        X = scans.Angle{angleID}.RotatedData.X;
        Y = scans.Angle{angleID}.RotatedData.Y;
        Z = scans.Angle{angleID}.RotatedData.Z;
        
        R = colsStep*angleID;
        
        plot3(X, Y, Z, '.','Color',[R,0,1-R], 'Parent', mainAxes);
    end
    
    lidarObj = plot3(0, 0, 0, '.', 'Color', [0, 0.9, 0], 'MarkerSize', ...
        15, 'Parent', mainAxes);
    
    legend(mainAxes, lidarObj, 'LiDAR');
    
    hold(mainAxes,'off');
    axis(mainAxes,'equal');
    grid(mainAxes,'on');
    mainAxes.XLabel.String = 'X (mm)';
    mainAxes.YLabel.String = 'Y (mm)';
    mainAxes.ZLabel.String = 'Z (mm)';
end