function main()
%MAIN Main code to visualize all the experimental data
    datasetNames = {'Stairs.mat','StaticObjectScans.mat','Trajectories.mat'};
    
    [datasets, errorStr, areaMap] = loadExperimentalData(datasetNames);
    
    if ~isempty(errorStr)
        error(errorStr);
    end
    generateGUI(datasets, areaMap);
end

function [datasets, errorStr, areaMap] = loadExperimentalData(dsNames)
    numDatasets = length(dsNames);
    datasets = cell(1,numDatasets);

    [areaMap, errorStr] = loadMap('AreaMap.mat');
    if ~isempty(errorStr)
        return;
    end
    
    for datasetID = 1:numDatasets
        try
            dataset = [];
            load([pwd, '\ExperimentalData\', dsNames{datasetID}]);
            datasets{datasetID} = dataset;
        catch
            st = dbstack;
            errorStr = ['Dataset "', dsNames{datasetID}, '" not found. Make sure ',...
                '"ExperimentalData" folder is located under the same ', char(10),...
                'directory than ', st(1).file];
            return;
        end
    end
end

function [map, errorStr] = loadMap(mapName)
    map = [];
    errorStr = [];
    try
        load([pwd, '\ExperimentalData\',mapName]);
    catch
        st = dbstack;
        errorStr = ['Map "', mapName, '" not found. Make sure ',...
            '"ExperimentalData" folder is located under the same ', char(10),...
            'directory than ', st(1).file];
    end
end

function generateGUI(datasets,areaMap)
    mainWindow = figure;
    mainWindow.MenuBar = 'none';
    mainAxis = axes;
    
    createShowMapMenu(areaMap, mainWindow, mainAxis);
    createTrajectoriesMenu(datasets{3}, mainWindow, mainAxis);
    createFirstScanMenu(datasets{3}, mainWindow, mainAxis);
    create3DReconstructionMenu(datasets{2}, mainWindow, mainAxis);
    createFirstScanProjMenu(datasets{1}, mainWindow, mainAxis);
end

function mainWindow = createShowMapMenu(areaMap, mainWindow, mainAxis)
    mapMenu = uimenu(mainWindow,'Label','Map:');
    uimenu(mapMenu,'Label','Area Map', 'Callback', {@MapVisualizer,...
        mainAxis, areaMap, mainWindow});
end

function mainWindow = createTrajectoriesMenu(dataset, mainWindow, mainAxis)
    numTrajectories = length(dataset.Collection);
    expMenuObjList = cell(1,numTrajectories);
    
    expMenu = uimenu(mainWindow,'Label','Trajectories:');
    for trajID = 1:numTrajectories
        expMenuObjList{trajID} = uimenu(expMenu, 'Label', ...
            dataset.Collection{trajID}.Name,'Callback',{@TrajectoriesVisualizer,...
            dataset.Collection{trajID},dataset.AreaMap , mainAxis, mainWindow});
    end
end

function mainWindow = createFirstScanMenu(dataset, mainWindow, mainAxis)
    numTrajectories = length(dataset.Collection);
    expMenuObjList = cell(1,numTrajectories);

    expMenu = uimenu(mainWindow,'Label','First Scan:');
    for trajID = 1:numTrajectories
        expMenuObjList{trajID} = uimenu(expMenu,'Label',dataset.Collection{trajID}.Name,...
            'Callback',{@FirstScanVisualizer, dataset.Collection{trajID},...
            mainAxis, mainWindow});
    end
end

function mainWindow = create3DReconstructionMenu(dataset, mainWindow, mainAxis)
    numObjects = length(dataset.Collection);
    expMenuObjList = cell(1,numObjects);
    
    expMenu = uimenu(mainWindow,'Label','3D Reconstruction:');
    for objID = 1:numObjects
        expMenuObjList{objID} = uimenu(expMenu,'Label',dataset.Collection{objID}.Name);
        distancesList = dataset.Collection{objID}.Distances_List_mm;
        numDistances = length(distancesList);
        
        for distID = 1:numDistances
            if distancesList(distID) == -1
                tempMenu = uimenu(expMenuObjList{objID},'Label','No object');
            else
                tempMenu = uimenu(expMenuObjList{objID},'Label',[num2str(distancesList(distID)),' mm']);
            end
            
            tempMenu.Callback = {@StaticObjectsVisualizer, dataset.Collection{objID},...
                    distID, mainAxis, mainWindow};
        end
    end
end

function mainWindow = createFirstScanProjMenu(dataset, mainWindow, mainAxis)
    numScenes = length(dataset.Collection);
    expMenuObjList = cell(1,numScenes);
    
    expMenu = uimenu(mainWindow,'Label','First Scan Projection:');
    for sceneID = 1:numScenes
        expMenuObjList{sceneID} = uimenu(expMenu,'Label',dataset.Collection{sceneID}.Name);
        perspectiveList = {'2D View', '3D View', 'Projection'};
        
        for perspID = 1:length(perspectiveList)
            if perspID < 3
                uimenu(expMenuObjList{sceneID},'Label',perspectiveList{perspID},...
                    'Callback',{@FirstScanProjection, dataset.Collection{sceneID},...
                    mainAxis, mainWindow, perspID});
            else
                uimenu(expMenuObjList{sceneID},'Label',perspectiveList{perspID},...
                    'Callback',{@LidarProjectionVisualizer, dataset.Collection{sceneID},...
                    mainAxis, mainWindow});
            end
        end

    end
end